@@ -415,7 +415,7 @@ static int add_directory_items(struct btrfs_trans_handle *trans,
 
 	location.objectid = objectid;
 	location.offset = 0;
-	btrfs_set_key_type(&location, BTRFS_INODE_ITEM_KEY);
+	location.type = BTRFS_INODE_ITEM_KEY;
 
 	if (S_ISDIR(st->st_mode))
 		filetype = BTRFS_FT_DIR;
@@ -539,7 +539,6 @@ static int add_inode_items(struct btrfs_trans_handle *trans,
 			   int dir_index_cnt, struct btrfs_inode_item *inode_ret)
 {
 	int ret;
-	struct btrfs_key inode_key;
 	struct btrfs_inode_item btrfs_inode;
 	u64 objectid;
 	u64 inode_size = 0;
@@ -552,10 +551,6 @@ static int add_inode_items(struct btrfs_trans_handle *trans,
 		btrfs_set_stack_inode_size(&btrfs_inode, inode_size);
 	}
 
-	inode_key.objectid = objectid;
-	inode_key.offset = 0;
-	btrfs_set_key_type(&inode_key, BTRFS_INODE_ITEM_KEY);
-
 	ret = btrfs_insert_inode(trans, root, objectid, &btrfs_inode);
 
 	*inode_ret = btrfs_inode;
@@ -835,7 +830,7 @@ static int traverse_directory(struct btrfs_trans_handle *trans,
 
 	root_dir_key.objectid = btrfs_root_dirid(&root->root_item);
 	root_dir_key.offset = 0;
-	btrfs_set_key_type(&root_dir_key, BTRFS_INODE_ITEM_KEY);
+	root_dir_key.type = BTRFS_INODE_ITEM_KEY;
 	ret = btrfs_lookup_inode(trans, root, &path, &root_dir_key, 1);
 	if (ret) {
 		error("failed to lookup root dir: %d", ret);
@@ -905,8 +900,9 @@ static int traverse_directory(struct btrfs_trans_handle *trans,
 			if (ret == -EEXIST) {
 				if (st.st_nlink <= 1) {
 					error(
-			"item %s already exists but has wrong st_nlink %ld <= 1",
-						cur_file->d_name, st.st_nlink);
+			"item %s already exists but has wrong st_nlink %lu <= 1",
+						cur_file->d_name,
+						(unsigned long)st.st_nlink);
 					goto fail;
 				}
 				continue;
@@ -976,15 +972,6 @@ fail_no_dir:
 	goto out;
 }
 
-static int open_target(char *output_name)
-{
-	int output_fd;
-	output_fd = open(output_name, O_CREAT | O_RDWR,
-		         S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
-
-	return output_fd;
-}
-
 static int create_chunks(struct btrfs_trans_handle *trans,
 			 struct btrfs_root *root, u64 num_of_meta_chunks,
 			 u64 size_of_data,
@@ -1636,6 +1623,12 @@ int main(int argc, char **argv)
 				 features))
 		exit(1);
 
+	if (sectorsize < sizeof(struct btrfs_super_block)) {
+		error("sectorsize smaller than superblock: %u < %zu",
+				sectorsize, sizeof(struct btrfs_super_block));
+		exit(1);
+	}
+
 	/* Check device/block_count after the nodesize is determined */
 	if (block_count && block_count < btrfs_min_dev_size(nodesize)) {
 		error("size %llu is too small to make a usable filesystem",
@@ -1697,7 +1690,8 @@ int main(int argc, char **argv)
 			exit(1);
 		}
 	} else {
-		fd = open_target(file);
+		fd = open(file, O_CREAT | O_RDWR,
+				S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
 		if (fd < 0) {
 			error("unable to open %s: %s", file, strerror(errno));
 			exit(1);
